/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0 and the Server Side Public License, v 1; you may not use this file except
 * in compliance with, at your election, the Elastic License 2.0 or the Server
 * Side Public License, v 1.
 */

import {
  EuiButton,
  EuiButtonGroup,
  EuiCodeBlock,
  EuiEmptyPrompt,
  EuiFlexGroup,
  EuiFlexItem,
  EuiLoadingSpinner,
  EuiSpacer,
} from '@elastic/eui';
import { CONTROL_GROUP_TYPE } from '@kbn/controls-plugin/common';
import { CoreStart } from '@kbn/core/public';
import { DataViewsPublicPluginStart } from '@kbn/data-views-plugin/public';
import { ReactEmbeddableRenderer, ViewMode } from '@kbn/embeddable-plugin/public';
import { AggregateQuery, Filter, Query, TimeRange } from '@kbn/es-query';
import { PresentationContainer } from '@kbn/presentation-containers';
import {
  HasUniqueId,
  PublishesUnifiedSearch,
  PublishesViewMode,
  useStateFromPublishingSubject,
  ViewMode as ViewModeType,
} from '@kbn/presentation-publishing';
import { toMountPoint } from '@kbn/react-kibana-mount';
import React, { useEffect, useState } from 'react';
import useAsync from 'react-use/lib/useAsync';
import useMount from 'react-use/lib/useMount';
import { BehaviorSubject } from 'rxjs';
import { ControlGroupApi } from '../react_controls/control_group/types';

const toggleViewButtons = [
  {
    id: `viewModeToggle_edit`,
    value: ViewMode.EDIT,
    label: 'Edit mode',
  },
  {
    id: `viewModeToggle_view`,
    value: ViewMode.VIEW,
    label: 'View mode',
  },
];

/**
 * I am mocking the dashboard API so that the data table embeddble responds to changes to the
 * data view publishing subject from the control group
 */
type MockedDashboardApi = PresentationContainer &
  PublishesViewMode &
  PublishesUnifiedSearch & {
    publishFilters: (newFilters: Filter[] | undefined) => void;
    setViewMode: (newViewMode: ViewMode) => void;
    setChild: (child: HasUniqueId) => void;
  };

export const ReactControlExample = ({
  core,
  dataViews: dataViewsService,
}: {
  core: CoreStart;
  dataViews: DataViewsPublicPluginStart;
}) => {
  const [dashboardApi, setDashboardApi] = useState<MockedDashboardApi | undefined>(undefined);
  const [controlGroupApi, setControlGroupApi] = useState<ControlGroupApi | undefined>(undefined);
  const viewModeSelected = useStateFromPublishingSubject(dashboardApi?.viewMode);

  useMount(() => {
    const viewMode = new BehaviorSubject<ViewModeType>(ViewMode.EDIT as ViewModeType);
    const filters$ = new BehaviorSubject<Filter[] | undefined>([]);
    const query$ = new BehaviorSubject<Query | AggregateQuery | undefined>(undefined);
    const timeRange$ = new BehaviorSubject<TimeRange | undefined>(undefined);
    const children$ = new BehaviorSubject<{ [key: string]: unknown }>({});

    setDashboardApi({
      viewMode,
      filters$,
      query$,
      timeRange$,
      children$,
      publishFilters: (newFilters) => filters$.next(newFilters),
      setViewMode: (newViewMode) => viewMode.next(newViewMode),
      setChild: (child) => children$.next({ ...children$.getValue(), [child.uuid]: child }),
      removePanel: () => {},
      replacePanel: () => {
        return Promise.resolve('');
      },
      getPanelCount: () => {
        return 2;
      },
      addNewPanel: () => {
        return Promise.resolve(undefined);
      },
    });
  });

  // TODO: Maybe remove `useAsync` - see https://github.com/elastic/kibana/pull/182842#discussion_r1624909709
  const {
    loading,
    value: dataViews,
    error,
  } = useAsync(async () => {
    return await dataViewsService.find('kibana_sample_data_logs');
  }, []);

  useEffect(() => {
    if (!controlGroupApi) return;

    const subscription = controlGroupApi.filters$.subscribe((controlGroupFilters) => {
      if (dashboardApi) dashboardApi.publishFilters(controlGroupFilters);
    });

    return () => {
      subscription.unsubscribe();
    };
  }, [dashboardApi, controlGroupApi]);

  if (error || (!dataViews?.[0]?.id && !loading))
    return (
      <EuiEmptyPrompt
        iconType="error"
        color="danger"
        title={<h2>There was an error!</h2>}
        body={<p>{error ? error.message : 'Please add at least one data view.'}</p>}
      />
    );

  return loading ? (
    <EuiLoadingSpinner />
  ) : (
    <>
      <EuiFlexGroup>
        <EuiFlexItem grow={false}>
          <EuiButton
            fill
            onClick={() => {
              controlGroupApi?.onEdit();
            }}
            size="s"
          >
            Control group settings
          </EuiButton>
        </EuiFlexItem>
        <EuiFlexItem grow={false}>
          <EuiButton
            onClick={() => {
              core.overlays.openModal(
                toMountPoint(
                  <EuiCodeBlock language="json">
                    {JSON.stringify(controlGroupApi?.serializeState(), null, 2)}
                  </EuiCodeBlock>,
                  {
                    theme: core.theme,
                    i18n: core.i18n,
                  }
                )
              );
            }}
            size="s"
          >
            Serialize control group
          </EuiButton>
        </EuiFlexItem>
        <EuiFlexItem grow={false}>
          <EuiButtonGroup
            legend="Change the view mode"
            options={toggleViewButtons}
            idSelected={`viewModeToggle_${viewModeSelected}`}
            onChange={(_, value) => {
              dashboardApi?.setViewMode(value);
            }}
          />
        </EuiFlexItem>
      </EuiFlexGroup>
      <EuiSpacer size="m" />
      <ReactEmbeddableRenderer
        onApiAvailable={(api) => {
          dashboardApi?.setChild(api);
          setControlGroupApi(api as ControlGroupApi);
        }}
        hidePanelChrome={true}
        type={CONTROL_GROUP_TYPE}
        getParentApi={() => ({
          ...dashboardApi,
          getSerializedStateForChild: () => ({
            rawState: {
              controlStyle: 'oneLine',
              chainingSystem: 'HIERARCHICAL',
              showApplySelections: false,
              panelsJSON:
                '{"a957862f-beae-4f0c-8a3a-a6ea4c235651":{"type":"searchControl","order":0,"grow":true,"width":"medium","explicitInput":{"id":"a957862f-beae-4f0c-8a3a-a6ea4c235651","fieldName":"message","title":"Message","grow":true,"width":"medium","searchString": "this","enhancements":{}}}}',
              ignoreParentSettingsJSON:
                '{"ignoreFilters":false,"ignoreQuery":false,"ignoreTimerange":false,"ignoreValidations":false}',
            } as object,
            references: [
              {
                name: 'controlGroup_a957862f-beae-4f0c-8a3a-a6ea4c235651:searchControlDataView',
                type: 'index-pattern',
                id: dataViews?.[0].id!,
              },
            ],
          }),
        })}
        key={`control_group`}
      />
      <EuiSpacer size="l" />
      <div style={{ height: '400px' }}>
        <ReactEmbeddableRenderer
          type={'data_table'}
          getParentApi={() => ({
            ...dashboardApi,
            getSerializedStateForChild: () => ({
              rawState: {
                timeRange: { from: 'now-60d/d', to: 'now+60d/d' },
              },
              references: [],
            }),
          })}
          hidePanelChrome={false}
          onApiAvailable={(api) => {
            dashboardApi?.setChild(api);
          }}
        />
      </div>
    </>
  );
};
